import { MapModel } from '@/types/models';
import { numberToSafeInt } from '@/utils/number/numberToInt';
import { useCallback, useContext, useEffect } from 'react';
import {
  getScreenAspectRatios,
  getScreenDimension,
} from '@/components/Map/Drawer/ExportDrawer/ExportCompound/ImageSize/utils/useScreenAspectRatios';
import { ExportContext } from '../../ExportCompound.context';
import { DEFAULT_IMAGE_HEIGHT, DEFAULT_IMAGE_WIDTH } from '../ImageSize.constants';
import { ImageSize, ModelAspectRatios } from '../ImageSize.types';
import { useExportGraphicsSelectedModel } from './useExportGraphicsSelectedModel';
import { getModelAspectRatios } from './useModelAspectRatios';

interface UseImageSizeResults {
  handleChangeWidth(width: number): void;
  handleChangeHeight(height: number): void;
  width: number;
  height: number;
}

export const useImageSize = (): UseImageSizeResults => {
  const selectedModel = useExportGraphicsSelectedModel();
  const { data, setImageSize, setCurrentView } = useContext(ExportContext);
  const { imageSize, currentView } = data;
  let aspectRatios: ModelAspectRatios;
  if (currentView.value) {
    aspectRatios = getScreenAspectRatios();
  } else {
    aspectRatios = getModelAspectRatios(selectedModel);
  }

  const maxWidth = selectedModel?.width || DEFAULT_IMAGE_WIDTH;
  const maxHeight = selectedModel?.height || DEFAULT_IMAGE_HEIGHT;

  const getNormalizedImageSize = useCallback(
    (newImageSize: ImageSize): ImageSize => {
      const newWidth = newImageSize.width;
      const newHeight = newImageSize.height;

      const widthMinMax = currentView.value ? newWidth : Math.min(maxWidth, newWidth);
      const heightMinMax = currentView.value ? newHeight : Math.min(maxHeight, newHeight);

      const widthInt = numberToSafeInt(widthMinMax);
      const heightInt = numberToSafeInt(heightMinMax);

      return {
        width: widthInt,
        height: heightInt,
      };
    },
    [maxWidth, maxHeight],
  );

  const setDefaultModelImageSize = useCallback(
    (model: MapModel): void => {
      const screenDimension = getScreenDimension();
      const width = currentView.value ? screenDimension.width : model.width;
      const height = currentView.value ? screenDimension.height : model.height;
      const newImageSize = getNormalizedImageSize({
        width,
        height,
      });

      setImageSize(newImageSize);
    },
    [getNormalizedImageSize, setImageSize, setCurrentView, currentView],
  );

  const handleChangeWidth = (width: number): void => {
    const newImageSize = getNormalizedImageSize({
      width,
      height: width / aspectRatios.horizontal,
    });

    setImageSize(newImageSize);
  };

  const handleChangeHeight = (height: number): void => {
    const newImageSize = getNormalizedImageSize({
      height,
      width: height / aspectRatios.vertical,
    });

    setImageSize(newImageSize);
  };

  useEffect(() => {
    if (!selectedModel) {
      return;
    }

    setDefaultModelImageSize(selectedModel);
  }, [setDefaultModelImageSize, selectedModel, setCurrentView, currentView]);

  return {
    handleChangeWidth,
    handleChangeHeight,
    width: imageSize.width,
    height: imageSize.height,
  };
};
